-- active control structure example
--
-- from RTSS 94 paper of Elseaidy, Cleaveland and Baugh
--

-- Modeling:
--
-- Use 3 processes: a sensor, a controller, and an actuator
--
-- model two boolean variables as automata:
--	 SynchSensor (SS) and SycnhControl (SC)
--	 this allows use of asap semantics
-- model the V var and timer process of Cleaveland et al. using a timer
--	embedded in the controller
--
-- Notes
--
-- 1. stopwatches are used to model the timing enabling conditions
--      can make more computationally efficient by stopping them in certain
--	locations
-- 2. can use naive modeling of reads/writes of booleans, since there is a 
--	single reader/writer of each var for each value
--
-- Specification: pulses occur periodically within given time interval
--

define(cutlow,36)
define(cuthi,146)

var
	x,		-- stopwatch in sensor
	y,		-- stopwatch in controller
	t,		-- timer in controller, checks period of 135
	z: clock;	-- stopwatch in actuator

	w: integrator;  -- for the specification

automaton sensor
synclabs: read_SS_eq_0, write_SS_1;
initially sample_up & x=0;

loc sample_up: while x<=55 wait {}
	when 50<=x & x<=55 do {x' = 0} goto sample_down;
	
loc sample_down: while x=0 wait {}
	when True goto send_up;

loc send_up: while x<=10 wait {}
	when x=10 sync write_SS_1 do {x' = 0} goto synch;

loc synch: while x<=5 wait {}
	when x=5 do {x' = 0} goto send_down;

loc send_down: while True wait {}
	when asap sync read_SS_eq_0 do {x' = 0} goto sample_up;
end

automaton controller
synclabs: read_SS_eq_1, write_SS_0, write_SC_1, read_SC_eq_0;
initially receive_down & y=0 & t>=135;

loc receive_down: while y<=10 wait {}
	when y=10 do {y' = 0} goto synch;

loc synch: while True wait {}
	when asap sync read_SS_eq_1 do {y' = 0} goto synch_wait;

loc synch_wait: while y<=5 wait {}
	when y=5 sync write_SS_0 do {y' = 0} goto receive_up;

loc receive_up: while y<=0 wait {}
	when True goto receive_up_wait;
	when t>=135 do {t' = 0} goto calc_up;

loc receive_up_wait: while y<=1 & t <=135 wait {}
	-- the 1 second delay has passed
	when y=1 goto update_up;
	-- the timer gets reset "first"
	when t=135 do {t' = 0} goto calc_up;

loc update_up: while y<=25 wait {}
	when 20<=y & y<=25 do {y' = 0} goto update_down;

loc update_down: while y=0 wait {}
	when True goto receive_down;

loc calc_up: while y<=45 wait {}
	when 40<=y & y <=45 do {y' = 0} goto calc_down;

loc calc_down: while y=0 wait {}
	when True goto send_pulse_up;
	
loc send_pulse_up: while y<=10 wait {}
	when y=10 sync write_SC_1 do {y' = 0} goto synch_xp;

loc synch_xp: while y<=5 wait {}
	when y=5 goto send_pulse_down;

loc send_pulse_down: while True wait {}
	when asap sync read_SC_eq_0 do {y' = 0} goto receive_down;

end


automaton actuator
synclabs: read_SC_eq_1, write_SC_0, pulse;
initially receive_pulse_up & z=0;

loc receive_pulse_up: while z<=10 wait {}
	when z=10 goto synch_xp;

loc synch_xp: while True wait {}
	when asap sync read_SC_eq_1 do {z' = 0} goto synch_xp_wait;

loc synch_xp_wait: while z<=5 wait {}
	when z=5 sync write_SC_0 do {z' = 0} goto receive_pulse_down;

loc receive_pulse_down: while z=0 wait {}
	when True sync pulse goto apply_pulse_up;

loc apply_pulse_up: while z<=30 wait {}
	when 25<=z & z<=30 do {z' = 0} goto apply_pulse_down;

loc apply_pulse_down: while z=0 wait {}
	when True goto receive_pulse_up;
end

automaton synch_sensor
synclabs: read_SS_eq_0, read_SS_eq_1, write_SS_0, write_SS_1;
initially zero & True;

loc zero:  while True wait {}
	when True sync read_SS_eq_0 goto zero;
	when True sync write_SS_0 goto zero;
	when True sync write_SS_1 goto one;

loc one: while True wait {}
	when True sync read_SS_eq_1 goto one;
	when True sync write_SS_0 goto zero;
	when True sync write_SS_1 goto one;
end

automaton synch_controller
synclabs: read_SC_eq_0, read_SC_eq_1, write_SC_0, write_SC_1;
initially zero & True;

loc zero: while True wait {}
	when True sync read_SC_eq_0 goto zero;
	when True sync write_SC_0 goto zero;
	when True sync write_SC_1 goto one;

loc one: while True wait {}
	when True sync read_SC_eq_1 goto one;
	when True sync write_SC_0 goto zero;
	when True sync write_SC_1 goto one;
end


automaton spec
synclabs: pulse;
initially idle & True;

loc idle: while True wait {dw=0} 
	when True sync pulse do {w' = 0} goto testing;

loc testing: while True wait {dw=1}
	when True sync pulse do {w' = 0} goto testing;
	when w<=cutlow sync pulse goto viol;
	when w>=cuthi goto viol;

loc viol: while True wait {dw=0}
end

-- analysis commands


-- analysis commands

var 
  init_reg, viol_reg: region;

init_reg := 	loc[sensor] = sample_up & x = 0 & 
		loc[controller] = receive_down & y=0 & t>=135 & 
		loc[actuator] = receive_pulse_up & z=0 & 
		loc[synch_sensor] = zero & 
		loc[synch_controller] = zero & loc[spec] = idle;

viol_reg :=  loc[spec] = viol;


if empty( reach forward from  init_reg endreach & viol_reg )
  then prints "Active structure controller meets periodic requirements";
  else prints "Active structure controller violates periodic requirements";
endif;


